@@ -46,7 +46,7 @@ def run(*args, **extra_env):
     return subprocess.call(list(args), env=env, cwd=cwd)
 
 
-SETUP_CMD = [PYTHON, 'setup.py', '--build-universal-binary']
+SETUP_CMD = [PYTHON, 'setup.py']
 
 
 def build_frozen_launcher(extra_include_dirs):
@@ -89,7 +89,7 @@ def build_frozen_tools(kitty_exe):
 
 def sanitize_source_folder(path: str) -> None:
     for q in walk(path):
-        if os.path.splitext(q)[1] not in ('.py', '.glsl', '.ttf', '.otf'):
+        if os.path.splitext(q)[1] not in ('.py', '.glsl', '.ttf', '.otf', '.json'):
             os.unlink(q)
 
 
