@@ -453,6 +453,11 @@ void _glfwInputColorScheme(GLFWColorScheme value, bool is_initial_value) {
     if (_glfw.callbacks.system_color_theme_change) _glfw.callbacks.system_color_theme_change(value, is_initial_value);
 }
 
+void _glfwInputClipboardLost(GLFWClipboardType which) {
+    if (_glfw.callbacks.clipboard_lost) _glfw.callbacks.clipboard_lost(which);
+}
+
+
 //////////////////////////////////////////////////////////////////////////
 //////                       GLFW internal API                      //////
 //////////////////////////////////////////////////////////////////////////
@@ -679,6 +684,13 @@ GLFWAPI void glfwSetIgnoreOSKeyboardProcessing(bool enabled) {
     _glfw.ignoreOSKeyboardProcessing = enabled;
 }
 
+GLFWAPI bool glfwGrabKeyboard(int grab) {
+    if (grab == 0 || grab == 1) {
+        if (_glfwPlatformGrabKeyboard(grab)) _glfw.keyboard_grabbed = grab;
+    }
+    return _glfw.keyboard_grabbed;
+}
+
 GLFWAPI int glfwGetInputMode(GLFWwindow* handle, int mode)
 {
     _GLFWwindow* window = (_GLFWwindow*) handle;
