@@ -1,13 +1,9 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 import sys
-from typing import (
-    List,
-    Optional,
-)
 
-from kitty.typing import BossType, TypedDict
+from kitty.typing_compat import BossType, TypedDict
 
 from ..tui.handler import result_handler
 
@@ -30,13 +26,20 @@ The name for this question. Used to store history of previous answers which can
 be used for completions and via the browse history readline bindings.
 
 
+--title --window-title
+The title for the window in which the question is displayed. Only implemented
+for yesno and choices types.
+
+
 --choice -c
 type=list
 dest=choices
 A choice for the choices type. Can be specified multiple times. Every choice has
-the syntax: ``letter[;color]:text``. Where :italic:`letter` is the accelerator key
-and :italic:`text` is the corresponding text. There can be an optional color
-specification after the letter to indicate what color it should be.
+the syntax: ``letter[;color]:text``, where :italic:`text` is the choice
+text and :italic:`letter` is the selection key. :italic:`letter` is a single letter
+belonging to :italic:`text`. This letter is highlighted within the choice text.
+There can be an optional color specification after the letter
+to indicate what color it should be.
 For example: :code:`y:Yes` and :code:`n;red:No`
 
 
@@ -62,15 +65,15 @@ The text in the message to be replaced by hidden text. The hidden text is read v
 
 
 class Response(TypedDict):
-    items: List[str]
-    response: Optional[str]
+    items: list[str]
+    response: str | None
 
-def main(args: List[str]) -> Response:
+def main(args: list[str]) -> Response:
     raise SystemExit('This must be run as kitten ask')
 
 
 @result_handler()
-def handle_result(args: List[str], data: Response, target_window_id: int, boss: BossType) -> None:
+def handle_result(args: list[str], data: Response, target_window_id: int, boss: BossType) -> None:
     if data['response'] is not None:
         func, *args = data['items']
         getattr(boss, func)(data['response'], *args)
