@@ -4,15 +4,16 @@ package icat
 
 import (
 	"fmt"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui/graphics"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/images"
+	"github.com/kovidgoyal/kitty/tools/utils/shm"
 	"image"
 	"image/gif"
-	"kitty/tools/tty"
-	"kitty/tools/tui/graphics"
-	"kitty/tools/utils"
-	"kitty/tools/utils/images"
-	"kitty/tools/utils/shm"
 
-	"github.com/disintegration/imaging"
+	"github.com/edwvee/exiffix"
+	"github.com/kovidgoyal/imaging"
 )
 
 var _ = fmt.Print
@@ -108,8 +109,8 @@ func scale_image(imgd *image_data) bool {
 	return false
 }
 
-func load_one_frame_image(ctx *images.Context, imgd *image_data, src *opened_input) (img image.Image, err error) {
-	img, err = imaging.Decode(src.file, imaging.AutoOrientation(true))
+func load_one_frame_image(imgd *image_data, src *opened_input) (img image.Image, err error) {
+	img, _, err = exiffix.Decode(src.file)
 	src.Rewind()
 	if err != nil {
 		return
@@ -123,6 +124,7 @@ func load_one_frame_image(ctx *images.Context, imgd *image_data, src *opened_inp
 }
 
 var debugprintln = tty.DebugPrintln
+var _ = debugprintln
 
 func (frame *image_frame) set_disposal(anchor_frame int, disposal byte) int {
 	anchor_frame, frame.compose_onto = images.SetGIFFrameDisposal(frame.number, anchor_frame, disposal)
@@ -162,7 +164,7 @@ func render_image_with_go(imgd *image_data, src *opened_input) (err error) {
 			return err
 		}
 	default:
-		img, err := load_one_frame_image(&ctx, imgd, src)
+		img, err := load_one_frame_image(imgd, src)
 		if err != nil {
 			return err
 		}
