@@ -7,17 +7,16 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/fs"
 	"os"
 	"path"
 	"path/filepath"
 	"strings"
 	"time"
 
-	"kitty/tools/config"
-	"kitty/tools/utils"
-	"kitty/tools/utils/paths"
-	"kitty/tools/utils/shlex"
+	"github.com/kovidgoyal/kitty/tools/config"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/paths"
+	"github.com/kovidgoyal/kitty/tools/utils/shlex"
 
 	"github.com/bmatcuk/doublestar/v4"
 	"golang.org/x/sys/unix"
@@ -169,7 +168,7 @@ func get_arcname(loc, dest, home string) (arcname string) {
 		arcname = dest
 	} else {
 		arcname = filepath.Clean(loc)
-		if filepath.HasPrefix(arcname, home) {
+		if strings.HasPrefix(arcname, home) {
 			ra, err := filepath.Rel(home, arcname)
 			if err == nil {
 				arcname = ra
@@ -241,29 +240,27 @@ func excluded(pattern, path string) bool {
 }
 
 func get_file_data(callback func(h *tar.Header, data []byte) error, seen map[file_unique_id]string, local_path, arcname string, exclude_patterns []string) error {
-	s, err := os.Lstat(local_path)
-	if err != nil {
+	var s unix.Stat_t
+	if err := unix.Lstat(local_path, &s); err != nil {
 		return err
 	}
-	u, ok := s.Sys().(unix.Stat_t)
 	cb := func(h *tar.Header, data []byte, arcname string) error {
 		h.Name = arcname
 		if h.Typeflag == tar.TypeDir {
 			h.Name = strings.TrimRight(h.Name, "/") + "/"
 		}
 		h.Size = int64(len(data))
-		h.Mode = int64(s.Mode().Perm())
-		h.ModTime = s.ModTime()
+		h.Mode = int64(s.Mode & 0777) // discard the setuid, setgid and sticky bits
+		h.ModTime = time.Unix(s.Mtim.Unix())
+		h.AccessTime = time.Unix(s.Atim.Unix())
+		h.ChangeTime = time.Unix(s.Ctim.Unix())
 		h.Format = tar.FormatPAX
-		if ok {
-			h.AccessTime = time.Unix(0, u.Atim.Nano())
-			h.ChangeTime = time.Unix(0, u.Ctim.Nano())
-		}
 		return callback(h, data)
 	}
 	// we only copy regular files, directories and symlinks
-	switch s.Mode().Type() {
-	case fs.ModeSymlink:
+	switch s.Mode & unix.S_IFMT {
+	case unix.S_IFBLK, unix.S_IFIFO, unix.S_IFCHR, unix.S_IFSOCK: // ignored
+	case unix.S_IFLNK: // symlink
 		target, err := os.Readlink(local_path)
 		if err != nil {
 			return err
@@ -275,7 +272,7 @@ func get_file_data(callback func(h *tar.Header, data []byte) error, seen map[fil
 		if err != nil {
 			return err
 		}
-	case fs.ModeDir:
+	case unix.S_IFDIR: // directory
 		local_path = filepath.Clean(local_path)
 		type entry struct {
 			path, arcname string
@@ -318,13 +315,10 @@ func get_file_data(callback func(h *tar.Header, data []byte) error, seen map[fil
 				}
 			}
 		}
-	case 0: // Regular file
-		fid := file_unique_id{dev: uint64(u.Dev), inode: uint64(u.Ino)}
+	case unix.S_IFREG: // Regular file
+		fid := file_unique_id{dev: uint64(s.Dev), inode: uint64(s.Ino)}
 		if prev, ok := seen[fid]; ok { // Hard link
-			err = cb(&tar.Header{Typeflag: tar.TypeLink, Linkname: prev}, nil, arcname)
-			if err != nil {
-				return err
-			}
+			return cb(&tar.Header{Typeflag: tar.TypeLink, Linkname: prev}, nil, arcname)
 		}
 		seen[fid] = arcname
 		data, err := os.ReadFile(local_path)
@@ -391,9 +385,20 @@ func (self *ConfigSet) line_handler(key, val string) error {
 func load_config(hostname_to_match string, username_to_match string, overrides []string, paths ...string) (*Config, []config.ConfigLine, error) {
 	ans := &ConfigSet{all_configs: []*Config{NewConfig()}}
 	p := config.ConfigParser{LineHandler: ans.line_handler}
-	err := p.LoadConfig("ssh.conf", paths, overrides)
+	err := p.LoadConfig("ssh.conf", paths, nil)
 	if err != nil {
 		return nil, nil, err
 	}
-	return config_for_hostname(hostname_to_match, username_to_match, ans), p.BadLines(), nil
+	final_conf := config_for_hostname(hostname_to_match, username_to_match, ans)
+	bad_lines := p.BadLines()
+	if len(overrides) > 0 {
+		h := final_conf.Hostname
+		override_parser := config.ConfigParser{LineHandler: final_conf.Parse}
+		if err = override_parser.ParseOverrides(overrides...); err != nil {
+			return nil, nil, err
+		}
+		bad_lines = append(bad_lines, override_parser.BadLines()...)
+		final_conf.Hostname = h
+	}
+	return final_conf, bad_lines, nil
 }
