@@ -10,9 +10,10 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"github.com/kovidgoyal/kitty"
 	"io"
 	"io/fs"
-	"kitty"
+	"maps"
 	"net/url"
 	"os"
 	"os/exec"
@@ -21,23 +22,23 @@ import (
 	"path"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"strconv"
 	"strings"
+	"syscall"
 	"time"
 
-	"kitty/tools/cli"
-	"kitty/tools/themes"
-	"kitty/tools/tty"
-	"kitty/tools/tui"
-	"kitty/tools/tui/loop"
-	"kitty/tools/tui/shell_integration"
-	"kitty/tools/utils"
-	"kitty/tools/utils/secrets"
-	"kitty/tools/utils/shlex"
-	"kitty/tools/utils/shm"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/themes"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/tui/shell_integration"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/secrets"
+	"github.com/kovidgoyal/kitty/tools/utils/shlex"
+	"github.com/kovidgoyal/kitty/tools/utils/shm"
 
-	"golang.org/x/exp/maps"
-	"golang.org/x/exp/slices"
 	"golang.org/x/sys/unix"
 )
 
@@ -65,14 +66,13 @@ func get_destination(hostname string) (username, hostname_for_match string) {
 	}
 	if !parsed && strings.Contains(hostname, "@") && hostname[0] != '@' {
 		_, hostname_for_match, _ = strings.Cut(hostname, "@")
-		parsed = true
 	}
 	return
 }
 
 func read_data_from_shared_memory(shm_name string) ([]byte, error) {
 	data, err := shm.ReadWithSizeAndUnlink(shm_name, func(s fs.FileInfo) error {
-		if stat, ok := s.Sys().(unix.Stat_t); ok {
+		if stat, ok := s.Sys().(syscall.Stat_t); ok {
 			if os.Getuid() != int(stat.Uid) || os.Getgid() != int(stat.Gid) {
 				return fmt.Errorf("Incorrect owner on SHM file")
 			}
@@ -116,9 +116,6 @@ func parse_kitten_args(found_extra_args []string, username, hostname_for_match s
 			}
 		}
 	}
-	if len(overrides) > 0 {
-		overrides = append([]string{"hostname " + username + "@" + hostname_for_match}, overrides...)
-	}
 	return
 }
 
@@ -155,7 +152,7 @@ func set_askpass() (need_to_request_data bool) {
 	sentinel_exists := err == nil
 	if sentinel_exists || GetSSHVersion().SupportsAskpassRequire() {
 		if !sentinel_exists {
-			os.WriteFile(sentinel, []byte{0}, 0o644)
+			_ = os.WriteFile(sentinel, []byte{0}, 0o644)
 		}
 		need_to_request_data = false
 	}
@@ -322,9 +319,13 @@ func make_tarfile(cd *connection_data, get_local_env func(string) (string, bool)
 		return nil
 
 	}
-	add_data(fe{"data.sh", utils.UnsafeStringToBytes(env_script)})
+	if err = add_data(fe{"data.sh", utils.UnsafeStringToBytes(env_script)}); err != nil {
+		return nil, err
+	}
 	if cd.script_type == "sh" {
-		add_data(fe{"bootstrap-utils.sh", shell_integration.Data()[path.Join("shell-integration/ssh/bootstrap-utils.sh")].Data})
+		if err = add_data(fe{"bootstrap-utils.sh", shell_integration.Data()[path.Join("shell-integration/ssh/bootstrap-utils.sh")].Data}); err != nil {
+			return nil, err
+		}
 	}
 	if ksi != "" {
 		for _, fname := range shell_integration.Data().FilesMatching(
@@ -411,7 +412,7 @@ func prepare_script(script string, replacements map[string]string) string {
 	if _, found := replacements["EXPORT_HOME_CMD"]; !found {
 		replacements["EXPORT_HOME_CMD"] = ""
 	}
-	keys := maps.Keys(replacements)
+	keys := utils.Keys(replacements)
 	for i, key := range keys {
 		keys[i] = "\\b" + key + "\\b"
 	}
@@ -524,16 +525,20 @@ func get_remote_command(cd *connection_data) error {
 	return nil
 }
 
+var debugprintln = tty.DebugPrintln
+var _ = debugprintln
+
 func drain_potential_tty_garbage(term *tty.Term) {
-	err := term.ApplyOperations(tty.TCSANOW, tty.SetNoEcho)
+	err := term.ApplyOperations(tty.TCSANOW, tty.SetRaw)
 	if err != nil {
 		return
 	}
-	canary, err := secrets.TokenBase64()
+	canary, err := secrets.TokenHex()
 	if err != nil {
 		return
 	}
-	dcs, err := tui.DCSToKitty("echo", canary+"\n\r")
+	dcs, err := tui.DCSToKitty("echo", canary)
+	q := utils.UnsafeStringToBytes(canary)
 	if err != nil {
 		return
 	}
@@ -541,19 +546,18 @@ func drain_potential_tty_garbage(term *tty.Term) {
 	if err != nil {
 		return
 	}
-	q := utils.UnsafeStringToBytes(canary)
 	data := make([]byte, 0)
 	give_up_at := time.Now().Add(2 * time.Second)
 	buf := make([]byte, 0, 8192)
 	for !bytes.Contains(data, q) {
 		buf = buf[:cap(buf)]
-		timeout := give_up_at.Sub(time.Now())
+		timeout := time.Until(give_up_at)
 		if timeout < 0 {
 			break
 		}
 		n, err := term.ReadWithTimeout(buf, timeout)
 		if err != nil {
-			return
+			break
 		}
 		data = append(data, buf[:n]...)
 	}
@@ -597,7 +601,7 @@ func run_ssh(ssh_args, server_args, found_extra_args []string) (rc int, err erro
 	defer func() {
 		if data_shm != nil {
 			data_shm.Close()
-			data_shm.Unlink()
+			_ = data_shm.Unlink()
 		}
 	}()
 	cmd := append([]string{SSHExe()}, ssh_args...)
@@ -703,9 +707,9 @@ func run_ssh(ssh_args, server_args, found_extra_args []string) (rc int, err erro
 		c.Stdout = &b
 		c.Stderr = os.Stderr
 		if err := c.Run(); err != nil {
-			return 1, fmt.Errorf("%s\nSetup of port forward in SSH ControlMaster failed with error: %w", string(b.Bytes()), err)
+			return 1, fmt.Errorf("%s\nSetup of port forward in SSH ControlMaster failed with error: %w", b.String(), err)
 		}
-		port, err := strconv.Atoi(strings.TrimSpace(string(b.Bytes())))
+		port, err := strconv.Atoi(strings.TrimSpace(b.String()))
 		if err != nil {
 			os.Stderr.Write(b.Bytes())
 			return 1, fmt.Errorf("Setup of port forward in SSH ControlMaster failed with error: invalid resolved port returned: %s", b.String())
@@ -727,14 +731,22 @@ func run_ssh(ssh_args, server_args, found_extra_args []string) (rc int, err erro
 	if err != nil {
 		return 1, err
 	}
-	restore_escape_codes := loop.RESTORE_PRIVATE_MODE_VALUES
+	restore_escape_codes := loop.RESTORE_PRIVATE_MODE_VALUES + loop.HANDLE_TERMIOS_SIGNALS.EscapeCodeToReset()
 	if escape_codes_to_set_colors != "" {
 		restore_escape_codes += "\x1b[#Q"
 	}
-	defer func() {
-		term.WriteAllString(restore_escape_codes)
-		term.RestoreAndClose()
-	}()
+	sigs := make(chan os.Signal, 8)
+	signal.Notify(sigs, unix.SIGINT, unix.SIGTERM)
+	cleaned_up := false
+	cleanup := func() {
+		if !cleaned_up {
+			_ = term.WriteAllString(restore_escape_codes)
+			term.RestoreAndClose()
+			signal.Reset()
+			cleaned_up = true
+		}
+	}
+	defer cleanup()
 	err = get_remote_command(&cd)
 	if err != nil {
 		return 1, err
@@ -746,8 +758,6 @@ func run_ssh(ssh_args, server_args, found_extra_args []string) (rc int, err erro
 	if err != nil {
 		return 1, err
 	}
-	sigs := make(chan os.Signal, 8)
-	signal.Notify(sigs, unix.SIGINT, unix.SIGTERM)
 
 	if !cd.request_data {
 		rq := fmt.Sprintf("id=%s:pwfile=%s:pw=%s", cd.replacements["REQUEST_ID"], cd.replacements["PASSWORD_FILENAME"], cd.replacements["DATA_PASSWORD"])
@@ -760,24 +770,24 @@ func run_ssh(ssh_args, server_args, found_extra_args []string) (rc int, err erro
 			}
 		}
 		if err != nil {
-			c.Process.Kill()
-			c.Wait()
+			_ = c.Process.Kill()
+			_ = c.Wait()
 			return 1, err
 		}
 	}
 	go func() {
-		_ = <-sigs
+		<-sigs
 		// ignore any interrupt and terminate signals as they will usually be sent to the ssh child process as well
 		// and we are waiting on that.
 	}()
 	err = c.Wait()
 	drain_potential_tty_garbage(term)
-	signal.Reset(unix.SIGINT, unix.SIGTERM)
 	if err != nil {
 		var exit_err *exec.ExitError
 		if errors.As(err, &exit_err) {
 			if state := exit_err.ProcessState.String(); state == "signal: interrupt" {
-				unix.Kill(os.Getpid(), unix.SIGINT)
+				cleanup()
+				_ = unix.Kill(os.Getpid(), unix.SIGINT)
 				// Give the signal time to be delivered
 				time.Sleep(20 * time.Millisecond)
 			}
@@ -829,7 +839,7 @@ func EntryPoint(parent *cli.Command) {
 func specialize_command(ssh *cli.Command) {
 	ssh.Usage = "arguments for the ssh command"
 	ssh.ShortDescription = "Truly convenient SSH"
-	ssh.HelpText = "The ssh kitten is a thin wrapper around the ssh command. It automatically enables shell integration on the remote host, re-uses existing connections to reduce latency, makes the kitty terminfo database available, etc. It's invocation is identical to the ssh command. For details on its usage, see :doc:`/kittens/ssh`."
+	ssh.HelpText = "The ssh kitten is a thin wrapper around the ssh command. It automatically enables shell integration on the remote host, re-uses existing connections to reduce latency, makes the kitty terminfo database available, etc. Its invocation is identical to the ssh command. For details on its usage, see :doc:`/kittens/ssh`."
 	ssh.IgnoreAllArgs = true
 	ssh.OnlyArgsAllowed = true
 	ssh.ArgCompleter = cli.CompletionForWrapper("ssh")
