@@ -7,10 +7,13 @@
 # kitty --replay-commands file.txt
 # will replay the commands and pause at the end waiting for user to press enter
 
+import json
 import sys
 from contextlib import suppress
 from typing import Any
 
+from .fast_data_types import TEXT_SIZE_CODE
+
 CSI = '\x1b['
 OSC = '\x1b]'
 
@@ -74,8 +77,8 @@ def screen_designate_charset(which: int, to: int) -> None:
     write(f'\x1b{w}{t}')
 
 
-def select_graphic_rendition(*a: int) -> None:
-    write(f'{CSI}{";".join(map(str, a))}m')
+def select_graphic_rendition(payload: str) -> None:
+    write(f'{CSI}{payload}m')
 
 
 def deccara(*a: int) -> None:
@@ -150,6 +153,10 @@ def screen_erase_in_line(how: int, private: bool) -> None:
     write(f'{CSI}{"?" if private else ""}{how}K')
 
 
+def screen_erase_characters(num: int) -> None:
+    write(f'{CSI}{num}X')
+
+
 def screen_delete_lines(num: int) -> None:
     write(f'{CSI}{num}M')
 
@@ -162,6 +169,10 @@ def screen_cursor_down(count: int) -> None:
     write(f'{CSI}{count}B')
 
 
+def screen_cursor_down1(count: int) -> None:
+    write(f'{CSI}{count}E')
+
+
 def screen_report_key_encoding_flags() -> None:
     write(f'{CSI}?u')
 
@@ -216,7 +227,11 @@ def report_device_attributes(mode: int, char: int) -> None:
         x += chr(char)
     if mode:
         x += str(mode)
-    write(f'{CSI}{x}c')
+    write(f'{x}c')
+
+
+def report_device_status(x: int, private: bool) -> None:
+    write(f'{CSI}{"?" if private else ""}{x}n')
 
 
 def screen_decsace(mode: int) -> None:
@@ -263,11 +278,32 @@ def clipboard_control(payload: str) -> None:
     write(f'{OSC}{payload}\x07')
 
 
+def multicell_command(payload: str) -> None:
+    c = json.loads(payload)
+    text = c.pop('', '')
+    m = ''
+    if (w := c.pop('width', None)) is not None and w > 0:
+        m += f'w={w}:'
+    if (s := c.pop('scale', None)) is not None and s > 1:
+        m += f's={s}:'
+    if (n := c.pop('subscale_n', None)) is not None and n > 0:
+        m += f'n={n}:'
+    if (d := c.pop('subscale_d', None)) is not None and d > 0:
+        m += f'd={d}:'
+    if (v := c.pop('vertical_align', None)) is not None and v > 0:
+        m += f'v={v}:'
+    if (h := c.pop('horizontal_align', None)) is not None and h > 0:
+        m += f'h={h}:'
+    if c:
+        raise Exception('Unknown keys in multicell_command: ' + ', '.join(c))
+    write(f'{OSC}{TEXT_SIZE_CODE};{m.rstrip(":")};{text}\a')
+
+
 def replay(raw: str) -> None:
-    specials = {
-        'draw', 'set_title', 'set_icon', 'set_dynamic_color', 'set_color_table_color',
-        'process_cwd_notification', 'clipboard_control', 'shell_prompt_marking'
-    }
+    specials = frozenset({
+        'draw', 'set_title', 'set_icon', 'set_dynamic_color', 'set_color_table_color', 'select_graphic_rendition',
+        'process_cwd_notification', 'clipboard_control', 'shell_prompt_marking', 'multicell_command',
+    })
     for line in raw.splitlines():
         if line.strip() and not line.startswith('#'):
             cmd, rest = line.partition(' ')[::2]
