@@ -7,26 +7,28 @@ import socket
 import sys
 import termios
 import time
+from collections.abc import Callable, Iterator, Sequence
 from contextlib import suppress
 from functools import partial
 from pprint import pformat
-from typing import IO, Callable, Dict, Iterator, Optional, Sequence, Set, TypeVar
+from typing import IO, TypeVar
 
 from kittens.tui.operations import colored, styled
 
 from .child import cmdline_of_pid
 from .cli import version
+from .colors import theme_colors
 from .constants import extensions_dir, is_macos, is_wayland, kitty_base_dir, kitty_exe, shell_path
-from .fast_data_types import Color, SingleKey, num_users, wayland_compositor_data
+from .fast_data_types import Color, SingleKey, current_fonts, glfw_get_system_color_theme, num_users, opengl_version_string, wayland_compositor_data
 from .options.types import Options as KittyOpts
-from .options.types import defaults
+from .options.types import defaults, secret_options
 from .options.utils import KeyboardMode, KeyDefinition
-from .rgb import color_as_sharp
+from .rgb import color_as_sharp, color_from_int
 from .types import MouseEvent, Shortcut, mod_to_names
 
 AnyEvent = TypeVar('AnyEvent', MouseEvent, Shortcut)
 Print = Callable[..., None]
-ShortcutMap = Dict[Shortcut, str]
+ShortcutMap = dict[Shortcut, str]
 
 
 def green(x: str) -> str:
@@ -45,7 +47,7 @@ def print_event(ev: str, defn: str, print: Print) -> None:
     print(f'\t{ev} →  {defn}')
 
 
-def print_mapping_changes(defns: Dict[str, str], changes: Set[str], text: str, print: Print) -> None:
+def print_mapping_changes(defns: dict[str, str], changes: set[str], text: str, print: Print) -> None:
     if changes:
         print(title(text))
         for k in sorted(changes):
@@ -53,7 +55,7 @@ def print_mapping_changes(defns: Dict[str, str], changes: Set[str], text: str, p
 
 
 def compare_maps(
-    final: Dict[AnyEvent, str], final_kitty_mod: int, initial: Dict[AnyEvent, str], initial_kitty_mod: int, print: Print, mode_name: str = ''
+    final: dict[AnyEvent, str], final_kitty_mod: int, initial: dict[AnyEvent, str], initial_kitty_mod: int, print: Print, mode_name: str = ''
 ) -> None:
     ei = {k.human_repr(initial_kitty_mod): v for k, v in initial.items()}
     ef = {k.human_repr(final_kitty_mod): v for k, v in final.items()}
@@ -69,7 +71,7 @@ def compare_maps(
 
 
 
-def compare_opts(opts: KittyOpts, print: Print) -> None:
+def compare_opts(opts: KittyOpts, global_shortcuts: dict[str, SingleKey] | None, print: Print) -> None:
     from .config import load_config
     print()
     print('Config options different from defaults:')
@@ -83,6 +85,9 @@ def compare_opts(opts: KittyOpts, print: Print) -> None:
     fmt = f'{{:{field_len:d}s}}'
     colors = []
     for f in changed_opts:
+        if f in secret_options:
+            print(title(f'{f}:'), 'REDACTED FOR SECURITY')
+            continue
         val = getattr(opts, f)
         if isinstance(val, dict):
             print(title(f'{f}:'))
@@ -101,6 +106,15 @@ def compare_opts(opts: KittyOpts, print: Print) -> None:
             else:
                 if f == 'kitty_mod':
                     print(fmt.format(f), '+'.join(mod_to_names(getattr(opts, f))))
+                elif f in ('wayland_titlebar_color', 'macos_titlebar_color'):
+                    if val == 0:
+                        cval = 'system'
+                    elif val == 1:
+                        cval = 'background'
+                    else:
+                        col = color_from_int(val >> 8)
+                        cval = color_as_sharp(col) + ' ' + styled('  ', bg=col)
+                    colors.append(fmt.format(f) + ' ' + cval)
                 else:
                     print(fmt.format(f), str(getattr(opts, f)))
 
@@ -124,6 +138,11 @@ def compare_opts(opts: KittyOpts, print: Print) -> None:
         initial = {as_sc(k, v[0]): as_str(v) for k, v in initial_.keymap.items()}
         final_ = opts.keyboard_modes.get(kmn, KeyboardMode(kmn))
         final = {as_sc(k, v[0]): as_str(v) for k, v in final_.keymap.items()}
+        if not kmn and global_shortcuts:
+            for action, sk in global_shortcuts.items():
+                sc = Shortcut((sk,))
+                if sc not in final:
+                    final[sc] = action
         compare_maps(final, opts.kitty_mod, initial, default_opts.kitty_mod, print, mode_name=kmn)
     new_keyboard_modes = set(opts.keyboard_modes) - set(default_opts.keyboard_modes)
     for kmn in new_keyboard_modes:
@@ -173,7 +192,7 @@ class IssueData:
             return char
 
     def parse_issue_file(self, issue_file: IO[str]) -> Iterator[str]:
-        last_char: Optional[str] = None
+        last_char: str | None = None
         while True:
             this_char = issue_file.read(1)
             if not this_char:
@@ -219,7 +238,7 @@ def compositor_name() -> str:
     return ans
 
 
-def debug_config(opts: KittyOpts) -> str:
+def debug_config(opts: KittyOpts, global_shortcuts: dict[str, SingleKey] | None = None) -> str:
     from io import StringIO
     out = StringIO()
     p = partial(print, file=out)
@@ -246,19 +265,28 @@ def debug_config(opts: KittyOpts) -> str:
             p(f.read().strip())
     if not is_macos:
         p('Running under:', green(compositor_name()))
+    p(green('OpenGL:'), opengl_version_string())
     p(green('Frozen:'), 'True' if getattr(sys, 'frozen', False) else 'False')
+    p(green('Fonts:'))
+    for k, font in current_fonts().items():
+        if hasattr(font, 'identify_for_debug'):
+            flines = font.identify_for_debug().splitlines()
+            p(yellow(f'{k.rjust(8)}:'), flines[0])
+            for fl in flines[1:]:
+                p(' ' * 9, fl)
     p(green('Paths:'))
     p(yellow('  kitty:'), os.path.realpath(kitty_exe()))
     p(yellow('  base dir:'), kitty_base_dir)
     p(yellow('  extensions dir:'), extensions_dir)
     p(yellow('  system shell:'), shell_path)
+    p(f'System color scheme: {green(glfw_get_system_color_theme())}. Applied color theme type: {yellow(theme_colors.applied_theme or "none")}')
     if opts.config_paths:
         p(green('Loaded config files:'))
         p(' ', '\n  '.join(opts.config_paths))
     if opts.config_overrides:
         p(green('Loaded config overrides:'))
         p(' ', '\n  '.join(opts.config_overrides))
-    compare_opts(opts, p)
+    compare_opts(opts, global_shortcuts, p)
     p()
     p(green('Important environment variables seen by the kitty process:'))
 
