@@ -8,35 +8,42 @@
 #include "wcswidth.h"
 #include "lineops.h"
 #include "charsets.h"
+#include "resize.h"
 #include <structmember.h>
-#include "ringbuf.h"
+#include "../3rdparty/ringbuf/ringbuf.h"
 
 extern PyTypeObject Line_Type;
 #define SEGMENT_SIZE 2048
 
 static void
-add_segment(HistoryBuf *self) {
-    self->num_segments += 1;
-    self->segments = realloc(self->segments, sizeof(HistoryBufSegment) * self->num_segments);
+add_segment(HistoryBuf *self, index_type num) {
+    self->segments = realloc(self->segments, sizeof(HistoryBufSegment) * (self->num_segments + num));
     if (self->segments == NULL) fatal("Out of memory allocating new history buffer segment");
-    HistoryBufSegment *s = self->segments + self->num_segments - 1;
     const size_t cpu_cells_size = self->xnum * SEGMENT_SIZE * sizeof(CPUCell);
     const size_t gpu_cells_size = self->xnum * SEGMENT_SIZE * sizeof(GPUCell);
-    s->cpu_cells = calloc(1, cpu_cells_size + gpu_cells_size + SEGMENT_SIZE * sizeof(LineAttrs));
-    if (!s->cpu_cells) fatal("Out of memory allocating new history buffer segment");
-    s->gpu_cells = (GPUCell*)(((uint8_t*)s->cpu_cells) + cpu_cells_size);
-    s->line_attrs = (LineAttrs*)(((uint8_t*)s->gpu_cells) + gpu_cells_size);
+    const size_t segment_size = cpu_cells_size + gpu_cells_size + SEGMENT_SIZE * sizeof(LineAttrs);
+    char *mem = calloc(num, segment_size);
+    if (!mem) fatal("Out of memory allocating new history buffer segment");
+    char *needs_free = mem;
+    for (HistoryBufSegment *s = self->segments + self->num_segments; s < self->segments + self->num_segments + num; s++, mem += segment_size) {
+        s->cpu_cells = (CPUCell*)mem;
+        s->gpu_cells = (GPUCell*)(((uint8_t*)s->cpu_cells) + cpu_cells_size);
+        s->line_attrs = (LineAttrs*)(((uint8_t*)s->gpu_cells) + gpu_cells_size);
+        s->mem = NULL;
+    }
+    self->segments[self->num_segments].mem = needs_free;
+    self->num_segments += num;
 }
 
 static void
 free_segment(HistoryBufSegment *s) {
-    free(s->cpu_cells); memset(s, 0, sizeof(HistoryBufSegment));
+    free(s->mem); zero_at_ptr(s);
 }
 
 static index_type
 segment_for(HistoryBuf *self, index_type y) {
     index_type seg_num = y / SEGMENT_SIZE;
-    while (UNLIKELY(seg_num >= self->num_segments && SEGMENT_SIZE * self->num_segments < self->ynum)) add_segment(self);
+    while (UNLIKELY(seg_num >= self->num_segments && SEGMENT_SIZE * self->num_segments < self->ynum)) add_segment(self, 1);
     if (UNLIKELY(seg_num >= self->num_segments)) fatal("Out of bounds access to history buffer line number: %u", y);
     return seg_num;
 }
@@ -114,7 +121,7 @@ pagerhist_clear(HistoryBuf *self) {
 }
 
 static HistoryBuf*
-create_historybuf(PyTypeObject *type, unsigned int xnum, unsigned int ynum, unsigned int pagerhist_sz) {
+create_historybuf(PyTypeObject *type, unsigned int xnum, unsigned int ynum, unsigned int pagerhist_sz, TextCache *tc) {
     if (xnum == 0 || ynum == 0) {
         PyErr_SetString(PyExc_ValueError, "Cannot create an empty history buffer");
         return NULL;
@@ -124,8 +131,9 @@ create_historybuf(PyTypeObject *type, unsigned int xnum, unsigned int ynum, unsi
         self->xnum = xnum;
         self->ynum = ynum;
         self->num_segments = 0;
-        add_segment(self);
-        self->line = alloc_line();
+        add_segment(self, 1);
+        self->text_cache = tc_incref(tc);
+        self->line = alloc_line(self->text_cache);
         self->line->xnum = xnum;
         self->pagerhist = alloc_pagerhist(pagerhist_sz);
     }
@@ -133,10 +141,13 @@ create_historybuf(PyTypeObject *type, unsigned int xnum, unsigned int ynum, unsi
 }
 
 static PyObject *
-new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
+new_history_object(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     unsigned int xnum = 1, ynum = 1, pagerhist_sz = 0;
     if (!PyArg_ParseTuple(args, "II|I", &ynum, &xnum, &pagerhist_sz)) return NULL;
-    HistoryBuf *ans = create_historybuf(type, xnum, ynum, pagerhist_sz);
+    TextCache *tc = tc_alloc();
+    if (!tc) return PyErr_NoMemory();
+    HistoryBuf *ans = create_historybuf(type, xnum, ynum, pagerhist_sz, tc);
+    tc_decref(tc);
     return (PyObject*)ans;
 }
 
@@ -146,6 +157,7 @@ dealloc(HistoryBuf* self) {
     for (size_t i = 0; i < self->num_segments; i++) free_segment(self->segments + i);
     free(self->segments);
     free_pagerhist(self);
+    tc_decref(self->text_cache);
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
@@ -158,22 +170,25 @@ index_of(HistoryBuf *self, index_type lnum) {
     return (self->start_of_data + idx) % self->ynum;
 }
 
+static bool
+hb_line_is_continued(HistoryBuf *self, index_type num) {
+    if (num == 0) {
+        size_t sz;
+        if (self->pagerhist && self->pagerhist->ringbuf && (sz = ringbuf_bytes_used(self->pagerhist->ringbuf)) > 0) {
+            size_t pos = ringbuf_findchr(self->pagerhist->ringbuf, '\n', sz - 1);
+            if (pos >= sz) return true;  // ringbuf does not end with a newline
+        }
+        return false;
+    }
+    return cpu_lineptr(self, num - 1)[self->xnum-1].next_char_was_wrapped;
+}
+
 static void
 init_line(HistoryBuf *self, index_type num, Line *l) {
     // Initialize the line l, setting its pointer to the offsets for the line at index (buffer position) num
     l->cpu_cells = cpu_lineptr(self, num);
     l->gpu_cells = gpu_lineptr(self, num);
     l->attrs = *attrptr(self, num);
-    if (num > 0) {
-        l->attrs.is_continued = gpu_lineptr(self, num - 1)[self->xnum-1].attrs.next_char_was_wrapped;
-    } else {
-        l->attrs.is_continued = false;
-        size_t sz;
-        if (self->pagerhist && self->pagerhist->ringbuf && (sz = ringbuf_bytes_used(self->pagerhist->ringbuf)) > 0) {
-            size_t pos = ringbuf_findchr(self->pagerhist->ringbuf, '\n', sz - 1);
-            if (pos >= sz) l->attrs.is_continued = true;  // ringbuf does not end with a newline
-        }
-    }
 }
 
 void
@@ -181,9 +196,14 @@ historybuf_init_line(HistoryBuf *self, index_type lnum, Line *l) {
     init_line(self, index_of(self, lnum), l);
 }
 
+bool
+historybuf_is_line_continued(HistoryBuf *self, index_type lnum) {
+    return hb_line_is_continued(self, index_of(self, lnum));
+}
+
 bool
 history_buf_endswith_wrap(HistoryBuf *self) {
-    return gpu_lineptr(self, index_of(self, 0))[self->xnum-1].attrs.next_char_was_wrapped;
+    return cpu_lineptr(self, index_of(self, 0))[self->xnum-1].next_char_was_wrapped;
 }
 
 CPUCell*
@@ -211,8 +231,10 @@ historybuf_clear(HistoryBuf *self) {
     pagerhist_clear(self);
     self->count = 0;
     self->start_of_data = 0;
-    for (size_t i = 1; i < self->num_segments; i++) free_segment(self->segments + i);
-    self->num_segments = 1;
+    for (size_t i = 0; i < self->num_segments; i++) free_segment(self->segments + i);
+    free(self->segments); self->segments = NULL;
+    self->num_segments = 0;
+    add_segment(self, 1);
 }
 
 static bool
@@ -259,33 +281,39 @@ static void
 pagerhist_push(HistoryBuf *self, ANSIBuf *as_ansi_buf) {
     PagerHistoryBuf *ph = self->pagerhist;
     if (!ph) return;
-    const GPUCell *prev_cell = NULL;
-    Line l = {.xnum=self->xnum};
+    Line l = {.xnum=self->xnum, .text_cache=self->text_cache};
     init_line(self, self->start_of_data, &l);
-    line_as_ansi(&l, as_ansi_buf, &prev_cell, 0, l.xnum, 0);
+    ANSILineState s = {.output_buf=as_ansi_buf};
+    as_ansi_buf->len = 0;
+    line_as_ansi(&l, &s, 0, l.xnum, 0, true);
     pagerhist_write_bytes(ph, (const uint8_t*)"\x1b[m", 3);
     if (pagerhist_write_ucs4(ph, as_ansi_buf->buf, as_ansi_buf->len)) {
         char line_end[2]; size_t num = 0;
         line_end[num++] = '\r';
-        if (!l.gpu_cells[l.xnum - 1].attrs.next_char_was_wrapped) line_end[num++] = '\n';
+        if (!l.cpu_cells[l.xnum - 1].next_char_was_wrapped) line_end[num++] = '\n';
         pagerhist_write_bytes(ph, (const uint8_t*)line_end, num);
     }
 }
 
 static index_type
-historybuf_push(HistoryBuf *self, ANSIBuf *as_ansi_buf) {
+historybuf_push(HistoryBuf *self, ANSIBuf *as_ansi_buf, bool *needs_clear) {
     index_type idx = (self->start_of_data + self->count) % self->ynum;
-    init_line(self, idx, self->line);
     if (self->count == self->ynum) {
         pagerhist_push(self, as_ansi_buf);
         self->start_of_data = (self->start_of_data + 1) % self->ynum;
-    } else self->count++;
+        *needs_clear = true;
+    } else {
+        self->count++;
+        *needs_clear = false;
+    }
     return idx;
 }
 
 void
 historybuf_add_line(HistoryBuf *self, const Line *line, ANSIBuf *as_ansi_buf) {
-    index_type idx = historybuf_push(self, as_ansi_buf);
+    bool needs_clear;
+    index_type idx = historybuf_push(self, as_ansi_buf, &needs_clear);
+    init_line(self, idx, self->line);
     copy_line(line, self->line);
     *attrptr(self, idx) = line->attrs;
 }
@@ -299,13 +327,6 @@ historybuf_pop_line(HistoryBuf *self, Line *line) {
     return true;
 }
 
-static void
-history_buf_set_last_char_as_continuation(HistoryBuf *self, index_type y, bool wrapped) {
-    if (self->count > 0) {
-        gpu_lineptr(self, index_of(self, y))[self->xnum-1].attrs.next_char_was_wrapped = wrapped;
-    }
-}
-
 static PyObject*
 line(HistoryBuf *self, PyObject *val) {
 #define line_doc "Return the line with line number val. This buffer grows upwards, i.e. 0 is the most recently added line"
@@ -321,9 +342,10 @@ static PyObject*
 __str__(HistoryBuf *self) {
     PyObject *lines = PyTuple_New(self->count);
     if (lines == NULL) return PyErr_NoMemory();
+    RAII_ANSIBuf(buf);
     for (index_type i = 0; i < self->count; i++) {
         init_line(self, index_of(self, i), self->line);
-        PyObject *t = line_as_unicode(self->line, false);
+        PyObject *t = line_as_unicode(self->line, false, &buf);
         if (t == NULL) { Py_CLEAR(lines); return NULL; }
         PyTuple_SET_ITEM(lines, i, t);
     }
@@ -347,13 +369,13 @@ push(HistoryBuf *self, PyObject *args) {
 static PyObject*
 as_ansi(HistoryBuf *self, PyObject *callback) {
 #define as_ansi_doc "as_ansi(callback) -> The contents of this buffer as ANSI escaped text. callback is called with each successive line."
-    Line l = {.xnum=self->xnum};
-    const GPUCell *prev_cell = NULL;
-    ANSIBuf output = {0};
+    Line l = {.xnum=self->xnum, .text_cache=self->text_cache};
+    ANSIBuf output = {0}; ANSILineState s = {.output_buf=&output};
     for(unsigned int i = 0; i < self->count; i++) {
         init_line(self, i, &l);
-        line_as_ansi(&l, &output, &prev_cell, 0, l.xnum, 0);
-        if (!l.gpu_cells[l.xnum - 1].attrs.next_char_was_wrapped) {
+        output.len = 0;
+        line_as_ansi(&l, &s, 0, l.xnum, 0, true);
+        if (!l.cpu_cells[l.xnum - 1].next_char_was_wrapped) {
             ensure_space_for(&output, buf, Py_UCS4, output.len + 1, capacity, 2048, false);
             output.buf[output.len++] = '\n';
         }
@@ -370,9 +392,6 @@ end:
     Py_RETURN_NONE;
 }
 
-static Line*
-get_line(HistoryBuf *self, index_type y, Line *l) { init_line(self, index_of(self, self->count - y - 1), l); return l; }
-
 static char_type
 pagerhist_remove_char(PagerHistoryBuf *ph, unsigned *count, uint8_t record[8]) {
     uint32_t codep; UTF8State state = UTF8_ACCEPT;
@@ -392,7 +411,7 @@ static void
 pagerhist_rewrap_to(HistoryBuf *self, index_type cells_in_line) {
     PagerHistoryBuf *ph = self->pagerhist;
     if (!ph->ringbuf || !ringbuf_bytes_used(ph->ringbuf)) return;
-    PagerHistoryBuf *nph = calloc(sizeof(PagerHistoryBuf), 1);
+    PagerHistoryBuf *nph = calloc(1, sizeof(PagerHistoryBuf));
     if (!nph) return;
     nph->maximum_size = ph->maximum_size;
     nph->ringbuf = ringbuf_new(MIN(ph->maximum_size, ringbuf_capacity(ph->ringbuf) + 4096));
@@ -498,6 +517,12 @@ typedef struct {
     HistoryBuf *self;
 } GetLineWrapper;
 
+static Line*
+get_line(HistoryBuf *self, index_type y, Line *l) {
+    init_line(self, index_of(self, self->count - y - 1), l);
+    return l;
+}
+
 static Line*
 get_line_wrapper(void *x, int y) {
     GetLineWrapper *glw = x;
@@ -509,6 +534,7 @@ PyObject*
 as_text_history_buf(HistoryBuf *self, PyObject *args, ANSIBuf *output) {
     GetLineWrapper glw = {.self=self};
     glw.line.xnum = self->xnum;
+    glw.line.text_cache = self->text_cache;
     PyObject *ans = as_text_generic(args, &glw, get_line_wrapper, self->count, output, true);
     return ans;
 }
@@ -534,6 +560,24 @@ pagerhist_rewrap(HistoryBuf *self, PyObject *xnum) {
     Py_RETURN_NONE;
 }
 
+static PyObject*
+is_continued(HistoryBuf *self, PyObject *val) {
+#define is_continued_doc "is_continued(y) -> Whether the line y is continued or not"
+    unsigned long y = PyLong_AsUnsignedLong(val);
+    if (y >= self->count) { PyErr_SetString(PyExc_ValueError, "Out of bounds."); return NULL; }
+    index_type num = index_of(self, self->count - y - 1);
+    if (hb_line_is_continued(self, num)) { Py_RETURN_TRUE; }
+    Py_RETURN_FALSE;
+}
+
+static PyObject*
+endswith_wrap(HistoryBuf *self, PyObject *val UNUSED) {
+#define endswith_wrap_doc "endswith_wrap() -> Whether the last line is wrapped at the end of the buffer"
+    if (history_buf_endswith_wrap(self)) { Py_RETURN_TRUE; }
+    Py_RETURN_FALSE;
+}
+
+
 
 // Boilerplate {{{
 static PyObject* rewrap(HistoryBuf *self, PyObject *args);
@@ -541,6 +585,8 @@ static PyObject* rewrap(HistoryBuf *self, PyObject *args);
 
 static PyMethodDef methods[] = {
     METHOD(line, METH_O)
+    METHOD(is_continued, METH_O)
+    METHOD(endswith_wrap, METH_NOARGS)
     METHOD(as_ansi, METH_O)
     METHODB(pagerhist_write, METH_O),
     METHODB(pagerhist_rewrap, METH_O),
@@ -569,56 +615,79 @@ PyTypeObject HistoryBuf_Type = {
     .tp_methods = methods,
     .tp_members = members,
     .tp_str = (reprfunc)__str__,
-    .tp_new = new
+    .tp_new = new_history_object
 };
 
 INIT_TYPE(HistoryBuf)
 
-HistoryBuf *alloc_historybuf(unsigned int lines, unsigned int columns, unsigned int pagerhist_sz) {
-    return create_historybuf(&HistoryBuf_Type, columns, lines, pagerhist_sz);
+HistoryBuf *alloc_historybuf(unsigned int lines, unsigned int columns, unsigned int pagerhist_sz, TextCache *tc) {
+    return create_historybuf(&HistoryBuf_Type, columns, lines, pagerhist_sz, tc);
 }
+
 // }}}
 
-#define BufType HistoryBuf
-
-#define map_src_index(y) ((src->start_of_data + y) % src->ynum)
-
-#define init_src_line(src_y) init_line(src, map_src_index(src_y), src->line);
-
-#define next_dest_line(cont) { history_buf_set_last_char_as_continuation(dest, 0, cont); LineAttrs *lap = attrptr(dest, historybuf_push(dest, as_ansi_buf)); *lap = src->line->attrs; }
-
-#define first_dest_line next_dest_line(false);
-
-#include "rewrap.h"
-
-void
-historybuf_rewrap(HistoryBuf *self, HistoryBuf *other, ANSIBuf *as_ansi_buf) {
-    while(other->num_segments < self->num_segments) add_segment(other);
-    if (other->xnum == self->xnum && other->ynum == self->ynum) {
-        // Fast path
-        for (index_type i = 0; i < self->num_segments; i++) {
-            memcpy(other->segments[i].cpu_cells, self->segments[i].cpu_cells, SEGMENT_SIZE * self->xnum * sizeof(CPUCell));
-            memcpy(other->segments[i].gpu_cells, self->segments[i].gpu_cells, SEGMENT_SIZE * self->xnum * sizeof(GPUCell));
-            memcpy(other->segments[i].line_attrs, self->segments[i].line_attrs, SEGMENT_SIZE * sizeof(LineAttrs));
-        }
-        other->count = self->count; other->start_of_data = self->start_of_data;
-        return;
-    }
-    if (other->pagerhist && other->xnum != self->xnum && ringbuf_bytes_used(other->pagerhist->ringbuf))
-        other->pagerhist->rewrap_needed = true;
-    other->count = 0; other->start_of_data = 0;
+static void
+history_buf_set_last_char_as_continuation(HistoryBuf *self, index_type y, bool wrapped) {
     if (self->count > 0) {
-        rewrap_inner(self, other, self->count, NULL, NULL, as_ansi_buf);
-        for (index_type i = 0; i < other->count; i++) attrptr(other, (other->start_of_data + i) % other->ynum)->has_dirty_text = true;
+        cpu_lineptr(self, index_of(self, y))[self->xnum-1].next_char_was_wrapped = wrapped;
     }
 }
 
+index_type
+historybuf_next_dest_line(HistoryBuf *self, ANSIBuf *as_ansi_buf, Line *src_line, index_type dest_y, Line *dest_line, bool continued) {
+    history_buf_set_last_char_as_continuation(self, 0, continued);
+    bool needs_clear;
+    index_type idx = historybuf_push(self, as_ansi_buf, &needs_clear);
+    *attrptr(self, idx) = src_line->attrs;
+    init_line(self, idx, dest_line);
+    if (needs_clear) {
+        zero_at_ptr_count(dest_line->cpu_cells, dest_line->xnum);
+        zero_at_ptr_count(dest_line->gpu_cells, dest_line->xnum);
+    }
+    return dest_y + 1;
+}
+
+HistoryBuf*
+historybuf_alloc_for_rewrap(unsigned int columns, HistoryBuf *self) {
+    if (!self) return NULL;
+    HistoryBuf *ans = alloc_historybuf(self->ynum, columns, 0, self->text_cache);
+    if (ans) {
+        if (ans->num_segments < self->num_segments) add_segment(ans, self->num_segments - ans->num_segments);
+        ans->count = 0; ans->start_of_data = 0;
+    }
+    return ans;
+}
+
+void
+historybuf_finish_rewrap(HistoryBuf *dest, HistoryBuf *src) {
+    for (index_type i = 0; i < dest->count; i++) attrptr(dest, (dest->start_of_data + i) % dest->ynum)->has_dirty_text = true;
+    dest->pagerhist = src->pagerhist; src->pagerhist = NULL;
+    if (dest->pagerhist && dest->xnum != src->xnum && ringbuf_bytes_used(dest->pagerhist->ringbuf)) dest->pagerhist->rewrap_needed = true;
+}
+
+void
+historybuf_fast_rewrap(HistoryBuf *dest, HistoryBuf *src) {
+    for (index_type i = 0; i < src->num_segments; i++) {
+        memcpy(dest->segments[i].cpu_cells, src->segments[i].cpu_cells, SEGMENT_SIZE * src->xnum * sizeof(CPUCell));
+        memcpy(dest->segments[i].gpu_cells, src->segments[i].gpu_cells, SEGMENT_SIZE * src->xnum * sizeof(GPUCell));
+        memcpy(dest->segments[i].line_attrs, src->segments[i].line_attrs, SEGMENT_SIZE * sizeof(LineAttrs));
+    }
+    dest->count = src->count; dest->start_of_data = src->start_of_data;
+}
+
+
 static PyObject*
 rewrap(HistoryBuf *self, PyObject *args) {
-    HistoryBuf *other;
-    if (!PyArg_ParseTuple(args, "O!", &HistoryBuf_Type, &other)) return NULL;
+    unsigned xnum;
+    if (!PyArg_ParseTuple(args, "I", &xnum)) return NULL;
     ANSIBuf as_ansi_buf = {0};
-    historybuf_rewrap(self, other, &as_ansi_buf);
+    LineBuf *dummy = alloc_linebuf(4, self->xnum, self->text_cache);
+    if (!dummy) return PyErr_NoMemory();
+    RAII_PyObject(cleanup, (PyObject*)dummy); (void)cleanup;
+    TrackCursor cursors[1] = {{.is_sentinel=true}};
+    ResizeResult r = resize_screen_buffers(dummy, self, 8, xnum, &as_ansi_buf, cursors);
     free(as_ansi_buf.buf);
-    Py_RETURN_NONE;
+    if (!r.ok) return PyErr_NoMemory();
+    Py_CLEAR(r.lb);
+    return (PyObject*)r.hb;
 }
