@@ -2,12 +2,13 @@
 # License: GPLv3 Copyright: 2019, Kovid Goyal <kovid at kovidgoyal.net>
 
 import sys
+from collections.abc import Callable
 from contextlib import suppress
-from typing import Callable, Dict, Optional
+from typing import Optional
 
 from .constants import is_macos
 
-functional_key_name_aliases = {
+functional_key_name_aliases: dict[str, str] = {
     'ESC': 'ESCAPE',
     'PGUP': 'PAGE_UP',
     'PAGEUP': 'PAGE_UP',
@@ -24,13 +25,15 @@ functional_key_name_aliases = {
 }
 
 
-character_key_name_aliases: Dict[str, str] = {
+character_key_name_aliases: dict[str, str] = {
     'SPC': ' ',
     'SPACE': ' ',
     'STAR': '*',
     'MULTIPLY': '*',
     'PLUS': '+',
     'MINUS': '-',
+    'BAR': '|',
+    'PIPE': '|',
     'HYPHEN': '-',
     'EQUAL': '=',
     'UNDERSCORE': '_',
@@ -51,7 +54,7 @@ character_key_name_aliases: Dict[str, str] = {
 LookupFunc = Callable[[str, bool], Optional[int]]
 
 
-def null_lookup(name: str, case_sensitive: bool = False) -> Optional[int]:
+def null_lookup(name: str, case_sensitive: bool = False) -> int | None:
     return None
 
 
@@ -76,14 +79,14 @@ else:
         f.argtypes = [ctypes.c_char_p, ctypes.c_int]
         f.restype = ctypes.c_int
 
-        def xkb_lookup(name: str, case_sensitive: bool = False) -> Optional[int]:
+        def xkb_lookup(name: str, case_sensitive: bool = False) -> int | None:
             q = name.encode('utf-8')
             return f(q, int(case_sensitive)) or None
 
         return xkb_lookup
 
     def get_key_name_lookup() -> LookupFunc:
-        ans: Optional[LookupFunc] = getattr(get_key_name_lookup, 'ans', None)
+        ans: LookupFunc | None = getattr(get_key_name_lookup, 'ans', None)
         if ans is None:
             try:
                 ans = load_libxkb_lookup()
