@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
-from typing import Any, Collection, Dict, Generator, List, NamedTuple, Optional, Sequence, Tuple, Union
+from collections.abc import Collection, Generator, Sequence
+from typing import Any, NamedTuple, Optional, Union
 
 from kitty.borders import BorderColor
 from kitty.types import Edges, WindowGeometry
-from kitty.typing import EdgeLiteral, WindowType
+from kitty.typing_compat import EdgeLiteral, WindowType
 from kitty.window_list import WindowGroup, WindowList
 
 from .base import BorderLine, Layout, LayoutOpts, NeighborsMap, blank_rects_for_window, lgd, window_geometry_from_layouts
@@ -20,11 +21,11 @@ class Pair:
 
     def __init__(self, horizontal: bool = True):
         self.horizontal = horizontal
-        self.one: Optional[Union[Pair, int]] = None
-        self.two: Optional[Union[Pair, int]] = None
+        self.one: Pair | int | None = None
+        self.two: Pair | int | None = None
         self.bias = 0.5
         self.top = self.left = self.width = self.height = 0
-        self.between_borders: List[Edges] = []
+        self.between_borders: list[Edges] = []
         self.first_extent = self.second_extent = Extent()
 
     def __repr__(self) -> str:
@@ -141,31 +142,36 @@ class Pair:
             pair = self
             pair.horizontal = horizontal
             self.one, self.two = q
+            final_pair = pair
+
         else:
             pair = Pair(horizontal=horizontal)
             if self.one == existing_window_id:
                 self.one = pair
             else:
                 self.two = pair
-            tuple(map(pair.balanced_add, q))
-        return pair
+            for wid in q:
+                qp = pair.balanced_add(wid)
+                if wid == new_window_id:
+                    final_pair = qp
+        return final_pair
 
     def apply_window_geometry(
         self, window_id: int,
         window_geometry: WindowGeometry,
-        id_window_map: Dict[int, WindowGroup],
+        id_window_map: dict[int, WindowGroup],
         layout_object: Layout
     ) -> None:
         wg = id_window_map[window_id]
         wg.set_geometry(window_geometry)
         layout_object.blank_rects.extend(blank_rects_for_window(window_geometry))
 
-    def effective_border(self, id_window_map: Dict[int, WindowGroup]) -> int:
+    def effective_border(self, id_window_map: dict[int, WindowGroup]) -> int:
         for wid in self.all_window_ids():
             return id_window_map[wid].effective_border()
         return 0
 
-    def minimum_width(self, id_window_map: Dict[int, WindowGroup]) -> int:
+    def minimum_width(self, id_window_map: dict[int, WindowGroup]) -> int:
         if self.one is None or self.two is None or not self.horizontal:
             return lgd.cell_width
         bw = self.effective_border(id_window_map) if lgd.draw_minimal_borders else 0
@@ -180,7 +186,7 @@ class Pair:
             ans += lgd.cell_width
         return ans
 
-    def minimum_height(self, id_window_map: Dict[int, WindowGroup]) -> int:
+    def minimum_height(self, id_window_map: dict[int, WindowGroup]) -> int:
         if self.one is None or self.two is None or self.horizontal:
             return lgd.cell_height
         bw = self.effective_border(id_window_map) if lgd.draw_minimal_borders else 0
@@ -198,7 +204,7 @@ class Pair:
     def layout_pair(
         self,
         left: int, top: int, width: int, height: int,
-        id_window_map: Dict[int, WindowGroup],
+        id_window_map: dict[int, WindowGroup],
         layout_object: Layout
     ) -> None:
         self.between_borders = []
@@ -283,6 +289,10 @@ class Pair:
                 geom = window_geometry_from_layouts(xl, yl)
                 self.apply_window_geometry(self.two, geom, id_window_map, layout_object)
 
+    def set_bias(self, window_id: int, bias: int) -> None:
+        b = max(0, min(bias, 100)) / 100
+        self.bias = b if window_id == self.one else (1. - b)
+
     def modify_size_of_child(self, which: int, increment: float, is_horizontal: bool, layout_object: 'Splits') -> bool:
         if is_horizontal == self.horizontal and not self.is_redundant:
             if which == 2:
@@ -349,7 +359,7 @@ class Pair:
 
     def neighbors_for_window(self, window_id: int, ans: NeighborsMap, layout_object: 'Splits', all_windows: WindowList) -> None:
 
-        def quadrant(is_horizontal: bool, is_first: bool) -> Tuple[EdgeLiteral, EdgeLiteral]:
+        def quadrant(is_horizontal: bool, is_first: bool) -> tuple[EdgeLiteral, EdgeLiteral]:
             if is_horizontal:
                 if is_first:
                     return 'left', 'right'
@@ -371,7 +381,7 @@ class Pair:
                     ans[which].append(other)
 
         def is_neighbouring_geometry(a: WindowGeometry, b: WindowGeometry, direction: str) -> bool:
-            def edges(g: WindowGeometry) -> Tuple[int, int]:
+            def edges(g: WindowGeometry) -> tuple[int, int]:
                 return (g.top, g.bottom) if direction in ['left', 'right'] else (g.left, g.right)
 
             a1, a2 = edges(a)
@@ -418,12 +428,16 @@ class Pair:
 
 class SplitsLayoutOpts(LayoutOpts):
 
-    default_axis_is_horizontal: bool = True
+    default_axis_is_horizontal: bool | None = True
 
-    def __init__(self, data: Dict[str, str]):
-        self.default_axis_is_horizontal = data.get('split_axis', 'horizontal') == 'horizontal'
+    def __init__(self, data: dict[str, str]):
+        q = data.get('split_axis', 'horizontal')
+        if q == 'auto':
+            self.default_axis_is_horizontal = None
+        else:
+            self.default_axis_is_horizontal = q == 'horizontal'
 
-    def serialized(self) -> Dict[str, Any]:
+    def serialized(self) -> dict[str, Any]:
         return {'default_axis_is_horizontal': self.default_axis_is_horizontal}
 
 
@@ -434,14 +448,17 @@ class Splits(Layout):
     no_minimal_window_borders = True
 
     @property
-    def default_axis_is_horizontal(self) -> bool:
+    def default_axis_is_horizontal(self) -> bool | None:
         return self.layout_opts.default_axis_is_horizontal
 
     @property
     def pairs_root(self) -> Pair:
-        root: Optional[Pair] = getattr(self, '_pairs_root', None)
+        root: Pair | None = getattr(self, '_pairs_root', None)
         if root is None:
-            self._pairs_root = root = Pair(horizontal=self.default_axis_is_horizontal)
+            horizontal = self.default_axis_is_horizontal
+            if horizontal is None:
+                horizontal = True
+            self._pairs_root = root = Pair(horizontal=horizontal)
         return root
 
     @pairs_root.setter
@@ -483,7 +500,9 @@ class Splits(Layout):
         self,
         all_windows: WindowList,
         window: WindowType,
-        location: Optional[str]
+        location: str | None,
+        bias: float | None = None,
+        next_to: WindowType | None = None,
     ) -> None:
         horizontal = self.default_axis_is_horizontal
         after = True
@@ -493,21 +512,26 @@ class Splits(Layout):
             horizontal = False
         elif location in ('before', 'first'):
             after = False
-        aw = all_windows.active_window
-        if aw is not None:
-            ag = all_windows.active_group
-            assert ag is not None
+        aw = next_to or all_windows.active_window
+        if bias:
+            bias = max(0, min(abs(bias), 100)) / 100
+        if aw is not None and (ag := all_windows.group_for_window(aw)) is not None:
             group_id = ag.id
             pair = self.pairs_root.pair_for_window(group_id)
             if pair is not None:
-                if location == 'split':
+                if location == 'split' or horizontal is None:
                     wwidth = aw.geometry.right - aw.geometry.left
                     wheight = aw.geometry.bottom - aw.geometry.top
                     horizontal = wwidth >= wheight
                 target_group = all_windows.add_window(window, next_to=aw, before=not after)
-                pair.split_and_add(group_id, target_group.id, horizontal, after)
+                parent_pair = pair.split_and_add(group_id, target_group.id, horizontal, after)
+                if bias is not None:
+                    parent_pair.bias = bias if parent_pair.one == target_group.id else (1 - bias)
                 return
         all_windows.add_window(window)
+        p = self.pairs_root.balanced_add(window.id)
+        if bias is not None:
+            p.bias = bias
 
     def modify_size_of_window(
         self,
@@ -558,6 +582,17 @@ class Splits(Layout):
             pair.neighbors_for_window(wg.id, ans, self, all_windows)
         return ans
 
+    def move_window(self, all_windows: WindowList, delta: int = 1) -> bool:
+        before = all_windows.active_group
+        if before is None:
+            return False
+        before_idx = all_windows.active_group_idx
+        moved = super().move_window(all_windows, delta)
+        after = all_windows.groups[before_idx]
+        if moved and before.id != after.id:
+            self.pairs_root.swap_windows(before.id, after.id)
+        return moved
+
     def move_window_to_group(self, all_windows: WindowList, group: int) -> bool:
         before = all_windows.active_group
         if before is None:
@@ -569,7 +604,7 @@ class Splits(Layout):
             self.pairs_root.swap_windows(before.id, after.id)
         return moved
 
-    def layout_action(self, action_name: str, args: Sequence[str], all_windows: WindowList) -> Optional[bool]:
+    def layout_action(self, action_name: str, args: Sequence[str], all_windows: WindowList) -> bool | None:
         if action_name == 'rotate':
             args = args or ('90',)
             try:
@@ -590,28 +625,57 @@ class Splits(Layout):
                         pair.one, pair.two = pair.two, pair.one
                     return True
         elif action_name == 'move_to_screen_edge':
-            args = args or ('left',)
-            which = args[0]
-            horizontal = which in ('left', 'right')
+            count = 0
+            for wid in self.pairs_root.all_window_ids():
+                count += 1
+                if count > 2:
+                    break
+            if count > 1:
+                args = args or ('left',)
+                which = args[0]
+                horizontal = which in ('left', 'right')
+                wg = all_windows.active_group
+                if wg is not None:
+                    if count == 2:  # special case, a single split
+                        pair = self.pairs_root.pair_for_window(wg.id)
+                        if pair is not None:
+                            pair.horizontal = horizontal
+                            if which in ('left', 'top'):
+                                if pair.one != wg.id:
+                                    pair.one, pair.two = pair.two, pair.one
+                                    pair.bias = 1. - pair.bias
+                            else:
+                                if pair.one == wg.id:
+                                    pair.one, pair.two = pair.two, pair.one
+                                    pair.bias = 1. - pair.bias
+                            return True
+                    else:
+                        self.remove_windows(wg.id)
+                        new_root = Pair(horizontal)
+                        if which in ('left', 'top'):
+                            new_root.balanced_add(wg.id)
+                            new_root.two = self.pairs_root
+                        else:
+                            new_root.one = self.pairs_root
+                            new_root.two = wg.id
+                        self.pairs_root = new_root
+                        return True
+        elif action_name == 'bias':
+            args = args or ('50',)
+            bias = int(args[0])
             wg = all_windows.active_group
             if wg is not None:
-                self.remove_windows(wg.id)
-                new_root = Pair(horizontal)
-                if which in ('left', 'top'):
-                    new_root.balanced_add(wg.id)
-                    new_root.two = self.pairs_root
-                else:
-                    new_root.one = self.pairs_root
-                    new_root.two = wg.id
-                self.pairs_root = new_root
-                return True
+                pair = self.pairs_root.pair_for_window(wg.id)
+                if pair is not None:
+                    pair.set_bias(wg.id, bias)
+                    return True
 
         return None
 
-    def layout_state(self) -> Dict[str, Any]:
+    def layout_state(self) -> dict[str, Any]:
 
-        def add_pair(p: Pair) -> Dict[str, Any]:
-            ans: Dict[str, Any] = {}
+        def add_pair(p: Pair) -> dict[str, Any]:
+            ans: dict[str, Any] = {}
             ans['horizontal'] = p.horizontal
             ans['bias'] = p.bias
             if isinstance(p.one, Pair):
