@@ -34,17 +34,16 @@ typedef struct {
 #endif
 #ifndef HAS_SIGNAL_FD
     int signal_fds[2];
-#else
-    sigset_t signals;
 #endif
+    sigset_t signals;
     int wakeup_read_fd;
     int signal_read_fd;
     int handled_signals[16];
     size_t num_handled_signals;
 } LoopData;
-typedef void(*handle_signal_func)(const siginfo_t* siginfo, void *data);
+typedef bool(*handle_signal_func)(const siginfo_t* siginfo, void *data);
 
-bool init_loop_data(LoopData *ld, size_t num_handled_signals, ...);
+bool init_loop_data(LoopData *ld, ...);
 void free_loop_data(LoopData *ld);
 void wakeup_loop(LoopData *ld, bool in_signal_handler, const char*);
 void read_signals(int fd, handle_signal_func callback, void *data);
