@@ -6,10 +6,10 @@
 
 
 import os
+from collections.abc import Callable, Sequence
 from concurrent.futures import ProcessPoolExecutor
-from multiprocessing import util
-from multiprocessing import context, get_all_start_methods, get_context, spawn
-from typing import Any, Callable, Sequence, Optional, Tuple, Union,  TYPE_CHECKING
+from multiprocessing import context, get_all_start_methods, get_context, spawn, util
+from typing import TYPE_CHECKING, Any, Union
 
 from .constants import kitty_exe
 
@@ -17,18 +17,11 @@ orig_spawn_passfds = util.spawnv_passfds
 orig_executable = spawn.get_executable()
 
 if TYPE_CHECKING:
-    from pickle import PickleBuffer
-    from array import array
-    from mmap import mmap
-    from ctypes import _CData
-    from typing import SupportsInt, SupportsIndex, Protocol
+    from typing import SupportsIndex, SupportsInt
 
-    class SupportsTrunc(Protocol):
-        def __trunc__(self) -> int: ...
+    from _typeshed import ReadableBuffer, SupportsTrunc
 
-    ArgsType = Sequence[
-        Union[str, Union[bytes, Union[bytearray, memoryview, array[Any], mmap, _CData, PickleBuffer]], SupportsInt, SupportsIndex, SupportsTrunc]
-    ]
+    ArgsType = Sequence[Union[str, ReadableBuffer, SupportsInt, SupportsIndex, SupportsTrunc]]
 else:
     ArgsType = Sequence[str]
 
@@ -58,12 +51,12 @@ def unmonkey_patch_multiprocessing() -> None:
 
 def get_process_pool_executor(
     prefer_fork: bool = False,
-    max_workers: Optional[int] = None,
-    initializer: Optional[Callable[..., None]] = None,
-    initargs: Tuple[Any, ...] = ()
+    max_workers: int | None = None,
+    initializer: Callable[..., None] | None = None,
+    initargs: tuple[Any, ...] = ()
 ) -> ProcessPoolExecutor:
     if prefer_fork and 'fork' in get_all_start_methods():
-        ctx: Union[context.DefaultContext, context.ForkContext] = get_context('fork')
+        ctx: context.DefaultContext | context.ForkContext = get_context('fork')
     else:
         monkey_patch_multiprocessing()
         ctx = get_context()
@@ -75,13 +68,25 @@ def get_process_pool_executor(
 
 def test_spawn() -> None:
     monkey_patch_multiprocessing()
+    import shutil
+    import subprocess
+    from queue import Empty
     try:
         from multiprocessing import get_context
         ctx = get_context('spawn')
         q = ctx.Queue()
         p = ctx.Process(target=q.put, args=('hello',))
         p.start()
-        x = q.get(timeout=8)
+        try:
+            x = q.get(timeout=8)
+        except Empty:
+            p.join()
+            rc = p.exitcode
+            if rc == 0:
+                raise TimeoutError('Timed out waiting for response from spawned process')
+            if shutil.which('coredumpctl'):
+                subprocess.run(['sh', '-c', 'echo bt | coredumpctl debug'])
+            raise SystemExit(f'Spawned process exited with return code: {rc}')
         assert x == 'hello'
         p.join()
     finally:
