@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2025, Kovid Goyal <kovid at kovidgoyal.net>
 
-# This module must be runnable by a vanilla python interperter
+# This module must be runnable by a vanilla python interpreter
 # as it is used to generate C code when building kitty
 
 import re
@@ -323,6 +323,14 @@ def generate_c_parsers() -> Iterator[str]:
 
 
 # kitty CLI spec {{{
+grab_keyboard_docs = """\
+Grab the keyboard. This means global shortcuts defined in the OS will be passed to kitty instead. Useful if
+you want to create an OS modal window. How well this
+works depends on the OS/window manager/desktop environment. On Wayland it works only if the compositor implements
+the :link:`inhibit-keyboard-shortcuts protocol <https://wayland.app/protocols/keyboard-shortcuts-inhibit-unstable-v1>`.
+On macOS Apple doesn't allow applications to grab the keyboard without special permissions, so it doesn't work.
+"""
+
 listen_on_defn = f'''\
 --listen-on
 completion=type:special group:complete_kitty_listen_on
@@ -341,6 +349,15 @@ is set to either: :code:`yes`, :code:`socket` or :code:`socket-only`. This can
 also be specified in :file:`kitty.conf`.
 '''
 
+wait_for_single_instance_defn = f'''\
+--wait-for-single-instance-window-close
+type=bool-set
+Normally, when using :option:`{appname} --single-instance`, :italic:`{appname}`
+will open a new window in an existing instance and quit immediately. With this
+option, it will not quit till the newly opened window is closed. Note that if no
+previous instance is found, then :italic:`{appname}` will wait anyway,
+regardless of this option.
+'''
 
 CONFIG_HELP = '''\
 Specify a path to the configuration file(s) to use. All configuration files are
@@ -456,13 +473,7 @@ Used in combination with the :option:`{appname} --single-instance` option. All
 :italic:`{appname}` instance within that group.
 
 
---wait-for-single-instance-window-close
-type=bool-set
-Normally, when using :option:`{appname} --single-instance`, :italic:`{appname}`
-will open a new window in an existing instance and quit immediately. With this
-option, it will not quit till the newly opened window is closed. Note that if no
-previous instance is found, then :italic:`{appname}` will wait anyway,
-regardless of this option.
+{wait_for_single_instance_defn}
 
 
 {listen_on_defn} To start in headless mode,
@@ -484,6 +495,11 @@ See also :opt:`remember_window_position` to have kitty automatically try
 to restore the previous window position.
 
 
+--grab-keyboard
+type=bool-set
+{grab_keyboard_docs}
+
+
 # Debugging options
 
 --version -v
@@ -539,8 +555,9 @@ type=bool-set
 '''
         setattr(kitty_options_spec, 'ans', OPTIONS.format(
             appname=appname, conf_name=appname, listen_on_defn=listen_on_defn,
-            config_help=CONFIG_HELP.format(appname=appname, conf_name=appname),
-        ))
+            grab_keyboard_docs=grab_keyboard_docs, wait_for_single_instance_defn=wait_for_single_instance_defn,
+            config_help=CONFIG_HELP.format(appname=appname, conf_name=appname
+        )))
     ans: str = getattr(kitty_options_spec, 'ans')
     return ans
 # }}}
@@ -613,7 +630,7 @@ The value :code:`center` anchors the panel to all sides and covers the entire
 display (on macOS the part of the display not covered by titlebar and dock).
 The panel can be shrunk and placed using the margin parameters.
 The value :code:`none` anchors the panel to the top left corner and should be
-placed and using the margin parameters. Its size is set by :option:`--lines`
+placed using the margin parameters. Its size is set by :option:`--lines`
 and :option:`--columns`. The value :code:`center-sized` is just like :code:`none` except
 that the panel is centered instead of in the top left corner and the margins have no effect.
 
@@ -643,7 +660,8 @@ Syntax: :italic:`name=value`. For example: :option:`kitty +kitten panel -o` font
 The panel can only be displayed on a single monitor (output) at a time. This allows
 you to specify which output is used, by name. If not specified the compositor will choose an
 output automatically, typically the last output the user interacted with or the primary monitor.
-Use the special value :code:`list` to get a list of available outputs.
+Use the special value :code:`list` to get a list of available outputs. Use :code:`listjson` for
+a json encoded output.
 
 
 --class --app-id
@@ -675,6 +693,11 @@ to :code:`on-demand`. Note that on Wayland, depending on the compositor, this ca
 becoming visible.
 
 
+--grab-keyboard
+type=bool-set
+{grab_keyboard_docs}
+
+
 --exclusive-zone
 type=int
 default={exclusive_zone}
@@ -709,6 +732,9 @@ panel invocations with the same :option:`--instance-group` will result
 in new panels being created in the first panel instance within that group.
 
 
+{wait_for_single_instance_defn}
+
+
 {listen_on_defn}
 
 
@@ -740,7 +766,17 @@ Path to a log file to store STDOUT/STDERR when using :option:`--detach`
 --debug-rendering
 type=bool-set
 For internal debugging use.
-'''.format(appname=appname, listen_on_defn=listen_on_defn, **d)
 
 
+--debug-input
+type=bool-set
+For internal debugging use.
+'''.format(
+    appname=appname, listen_on_defn=listen_on_defn, grab_keyboard_docs=grab_keyboard_docs,
+    wait_for_single_instance_defn=wait_for_single_instance_defn, **d)
+
+
+def panel_options_spec() -> str:
+    return build_panel_cli_spec(panel_defaults)
+
 # }}}
