@@ -1,9 +1,10 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2021, Kovid Goyal <kovid at kovidgoyal.net>
 
+from collections.abc import Callable, Iterator, Mapping, Sequence
 from enum import Enum
 from functools import update_wrapper
-from typing import TYPE_CHECKING, Any, Callable, Dict, Generic, Iterator, Mapping, NamedTuple, Optional, Sequence, Tuple, TypedDict, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Generic, NamedTuple, TypedDict, TypeVar, Union
 
 if TYPE_CHECKING:
     from kitty.fast_data_types import SingleKey
@@ -18,12 +19,12 @@ class SingleInstanceData(TypedDict):
     cwd: str
     session_data: str
     environ: Mapping[str, str]
-    notify_on_os_window_death: Optional[str]
+    notify_on_os_window_death: str | None
 
 
 class OverlayType(Enum):
-    transient: str = 'transient'
-    main: str = 'main'
+    transient = 'transient'
+    main = 'main'
 
 
 class ParsedShortcut(NamedTuple):
@@ -71,7 +72,17 @@ class LayerShellConfig(NamedTuple):
     edge: int = 0
     focus_policy: int = 0
     output_name: str = ''
-    size_in_cells: int = 0
+    x_size_in_pixels: int = 0
+    y_size_in_pixels: int = 0
+    x_size_in_cells: int = 0
+    y_size_in_cells: int = 0
+    requested_top_margin: int = 0
+    requested_left_margin: int = 0
+    requested_bottom_margin: int = 0
+    requested_right_margin: int = 0
+    requested_exclusive_zone: int = -1
+    override_exclusive_zone: bool = False
+    hide_on_focus_loss: bool = False
 
 
 def mod_to_names(mods: int, has_kitty_mod: bool = False, kitty_mod: int = 0) -> Iterator[str]:
@@ -95,7 +106,7 @@ def human_repr_of_single_key(self: 'SingleKey', kitty_mod: int) -> str:
 
 
 class Shortcut(NamedTuple):
-    keys: Tuple['SingleKey', ...]
+    keys: tuple['SingleKey', ...]
 
     def human_repr(self, kitty_mod: int = 0) -> str:
         return ' > '.join(human_repr_of_single_key(k, kitty_mod) for k in self.keys)
@@ -182,7 +193,7 @@ def run_once(f: Callable[[], _T]) -> 'RunOnce[_T]':
 
 
 @run_once
-def modmap() -> Dict[str, int]:
+def modmap() -> dict[str, int]:
     from .constants import is_macos
     from .fast_data_types import (
         GLFW_MOD_ALT,
