@@ -6,7 +6,9 @@
 
 #pragma once
 
-#include "kitty-uthash.h"
+#include <stdint.h>
+#include <stdbool.h>
+#include <stddef.h>
 
 typedef unsigned int window_logo_id_t;
 
@@ -15,12 +17,13 @@ typedef struct WindowLogo {
     bool load_from_disk_ok;
     uint32_t texture_id;
     uint8_t* bitmap;
+    size_t mmap_size;
 } WindowLogo;
 
 typedef struct WindowLogoTable WindowLogoTable;
 
 window_logo_id_t
-find_or_create_window_logo(WindowLogoTable *table, const char *path);
+find_or_create_window_logo(WindowLogoTable *table, const char *path, void *png_data, size_t png_data_size);
 
 WindowLogo*
 find_window_logo(WindowLogoTable *table, window_logo_id_t id);
