@@ -8,7 +8,7 @@ from . import BaseTest
 class TestSQP(BaseTest):
 
     def test_search_query_parser(self):
-        from kitty.search_query_parser import search, ParseException
+        from kitty.search_query_parser import ParseException, search
         locations = 'id'
         universal_set = {1, 2, 3, 4, 5}
 
