@@ -3,13 +3,12 @@
 package cli
 
 import (
-	"bufio"
 	"fmt"
-	"kitty/tools/cli/markup"
-	"kitty/tools/tty"
-	"kitty/tools/utils"
-	"kitty/tools/utils/style"
-	"kitty/tools/wcswidth"
+	"github.com/kovidgoyal/kitty/tools/cli/markup"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/style"
+	"github.com/kovidgoyal/kitty/tools/wcswidth"
 	"strings"
 )
 
@@ -38,7 +37,7 @@ func shell_input_parser(data []byte, shell_state map[string]string) ([][]string,
 	raw := string(data)
 	new_word := strings.HasSuffix(raw, "\n\n")
 	raw = strings.TrimRight(raw, "\n \t")
-	scanner := bufio.NewScanner(strings.NewReader(raw))
+	scanner := utils.NewLineScanner(raw)
 	words := make([]string, 0, 32)
 	for scanner.Scan() {
 		words = append(words, scanner.Text())
@@ -49,6 +48,9 @@ func shell_input_parser(data []byte, shell_state map[string]string) ([][]string,
 	return [][]string{words}, nil
 }
 
+var debugprintln = tty.DebugPrintln
+var _ = debugprintln
+
 func zsh_input_parser(data []byte, shell_state map[string]string) ([][]string, error) {
 	matcher := shell_state["_matcher"]
 	q := ""
@@ -76,7 +78,7 @@ func (self *Match) FormatForCompletionList(max_word_len int, f *markup.Context,
 		return word
 	}
 	word_len := wcswidth.Stringwidth(word)
-	line, _, _ := utils.Cut(strings.TrimSpace(desc), "\n")
+	line, _, _ := strings.Cut(strings.TrimSpace(desc), "\n")
 	desc = f.Prettify(line)
 
 	multiline := false
@@ -87,7 +89,7 @@ func (self *Match) FormatForCompletionList(max_word_len int, f *markup.Context,
 		word += strings.Repeat(" ", max_word_len-word_len)
 	}
 	if wcswidth.Stringwidth(desc) > max_desc_len {
-		desc = style.WrapTextAsLines(desc, "", max_desc_len-2)[0] + "…"
+		desc = style.WrapTextAsLines(desc, max_desc_len-2, style.WrapOptions{})[0] + "…"
 	}
 	if multiline {
 		return word + "\n" + strings.Repeat(" ", max_word_len+2) + desc
