@@ -5,8 +5,6 @@ package at
 import (
 	"fmt"
 	"strings"
-
-	"kitty/tools/utils"
 )
 
 var valid_color_names = map[string]bool{"active_fg": true, "active_bg": true, "inactive_fg": true, "inactive_bg": true}
@@ -14,7 +12,7 @@ var valid_color_names = map[string]bool{"active_fg": true, "active_bg": true, "i
 func parse_tab_colors(args []string) (map[string]any, error) {
 	ans := make(map[string]any, len(args))
 	for _, arg := range args {
-		key, val, found := utils.Cut(strings.ToLower(arg), "=")
+		key, val, found := strings.Cut(strings.ToLower(arg), "=")
 		if !found {
 			return nil, fmt.Errorf("%s is not a valid setting", arg)
 		}
