@@ -14,15 +14,16 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
 
-	"kitty/tools/utils"
-	"kitty/tools/utils/shm"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/shm"
 
-	"github.com/disintegration/imaging"
-	"golang.org/x/exp/slices"
+	"github.com/edwvee/exiffix"
+	"github.com/kovidgoyal/imaging"
 )
 
 var _ = fmt.Print
@@ -254,7 +255,7 @@ func OpenNativeImageFromReader(f io.ReadSeeker) (ans *ImageData, err error) {
 			return nil, err
 		}
 	} else {
-		img, err := imaging.Decode(f, imaging.AutoOrientation(true))
+		img, _, err := exiffix.Decode(f)
 		if err != nil {
 			return nil, err
 		}
